include "scripting/coroutine.lua"

local mil_site_names = {}
local cs_fields = {}
local ia_plr = wl.Game().players[wl.Game().interactive_player]
local map = wl.Game().map

-- Gather the names of military sites of the tribe a player plays.
-- We gather only military sites which can have more than 1 soldier
-- The tables keys are the name of the building, assigned value is
-- the radius to look for enemies.
function get_mil_site_names()
   local buildings = ia_plr.tribe.buildings
   for i, b in ipairs(buildings) do
      if b.type_name == "militarysite" and 
            b.max_number_of_soldiers > 1 then
         -- Adding 2 is some kind of cheating...
         vicinity = buildings[i].conquers + 2
         mil_site_names[buildings[i].name] = vicinity
      end
   end
end

-- Once the settings are made through this script we do
-- not want to apply the settings again. This makes it 
-- possible to let the player adjust the settings.
function already_set(field)
   for f, conquers in pairs(cs_fields) do
      if f.immovable then
         if f == field then
            return true
         end
      end
   end
   return false
end

-- Remove construction sites from the table.
-- There can be three states:
-- 1. A CS was build -> no construction site anymore
-- 2. A CS get destroyed after build has started -> ashes
-- 3. A CS get destroyed before build has started -> no immovable
function clean_cs_fields()
   for field, name in pairs(cs_fields) do
      if not field.immovable then
         -- constr site destroyed before start building
         print("Removing destroyed", field, name)
         cs_fields[field] = nil
      elseif field.immovable.descr.type_name ~= "constructionsite" then
         -- Building was build or constr site turned to ashes
         print("Removing ashes or built", field, name)
         cs_fields[field] = nil
      end
   end
end

-- Are there enemies nearby?
function is_enemy_nearby(field, v)
   vicinity=field:region(v)
   for i, n in pairs(vicinity) do
      if n.owner ~= nil and n.owner ~= ia_plr then
         return true
      end
   end
   return false
end

-- Set soldiers depending wether enemies are nearby or not
function set_soldiers(field, building)
   if not is_enemy_nearby(field, mil_site_names[building]) then
      field.immovable.setting_soldier_preference = "rookies"
      field.immovable.setting_soldier_capacity = 1
   else
      field.immovable.setting_soldier_preference = "heroes"
   end
end

-- Main loop.
-- The sleep time may has to be adjusted
function set_constr_sites()
   while true do
      for name, conquers in pairs(mil_site_names) do
         cs = ia_plr:get_constructionsites(name)
         for i, cs in ipairs(cs) do
            if not already_set(cs.fields[1]) then
               print("Adding cs:", cs.fields[1], "Building", cs.building)
               cs_fields[cs.fields[1]] = name
            end
         end
      end
      for field, name in pairs(cs_fields) do
         set_soldiers(field, name)
      end
   sleep(10000)
   clean_cs_fields()
   end
end

get_mil_site_names()
run(set_constr_sites)
